VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "Engine"
Attribute VB_GlobalNameSpace = True
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Attribute VB_Ext_KEY = "SavedWithClassBuilder" ,"Yes"
Attribute VB_Ext_KEY = "Top_Level" ,"Yes"
Attribute VB_Ext_KEY = "Member0" ,"Displays"
Option Explicit

' API DECLARATIONS
Private Declare Function SetPixelV Lib "gdi32" (ByVal hdc As Long, ByVal X As Long, ByVal Y As Long, ByVal crColor As Long) As Long

' CLASS PUBLIC DECLARATIONS ...

' Available resolutions (640x480, 800x600)
Public Enum dxsResolution
    dxsResLo = 0
    dxsResHi = 1
End Enum

' Available display types
Public Enum dxsDisplayType
    dxsDTViewport = 0
    dxsDTPanel = 1
    dxsDTRadar = 2
End Enum

' Amount of redrawing necessary for a display
Public Enum dxsRedrawState
    dxsRSNone = 0
    dxsRSBackgroundTotal = 1
    dxsRSBackgroundpartial = 2
    dxsRSBackbufferTotal = 3
    dxsRSBackbufferpartial = 4
End Enum

' Scroll coordinate mode passed
Public Enum dxsScrollMode
    dxsAbsolute = 0
    dxsRelative = 1
End Enum

' Scale mode for display position retrieval
Public Enum dxsScaleMode
    dxsSMUnit = 0
    dxsSMPixel = 1
    dxsSMMap = 2
End Enum



' CLASS LOCAL DECLARATIONS ...
Private Type tAniSquare
    PositionH As Integer
    PositionV As Integer
End Type

' Tile: Holds information on a single tile within the tileset
Private Type dxsTile
    Top As Integer               ' Horizontal tile position within tileset
    Left As Integer              ' Vertical tile position within tileset
    AnimationFrames As Byte      ' Number of animation frames within tile
    AnimationRandom As Integer   ' Random propability for tile animation to start
    AnimationDelay As Integer    ' Delay between frames when showing animation
    Altitude As Byte             ' Altitude a sprite must have to be allowed to pass this tile
    Color As Long                ' Color code of tile, used for display in radar-type map
End Type

' Tileset: Holds information on the tileset currently in use
Private Type dxsTileset
    ddSurface As DirectDrawSurface2
    Tiles(255) As dxsTile
    TileSize As Long
End Type

' Mapsquare: Holds information on a single map square within the map
Private Type dxsMapSquare
    TileIndex As Integer
    AnimationFrame As Byte
    AnimationAdvance As Long
End Type

' Map: Holds information on the map currently in use
Private Type dxsMap
    TileWidth As Long
    TileHeight As Long
    Square() As dxsMapSquare
End Type

Private Local_hWndDisplay As Long                   ' Window handle of window used by dx
Private Local_FrameCount As Long                    ' Global frame count of engine
Private Local_DX As DirectDraw2                     ' DirectDraw instance
Private Local_DXSPrimary As DirectDrawSurface2      ' Primary surface
Private Local_PrimaryWidth As Integer               ' Width of primary surface
Private Local_PrimaryHeight As Integer              ' Height of primary surface
Private Local_DXSBackbuffer As DirectDrawSurface2   ' Backbuffer surface
Private Local_LastError As String                   ' Last error occured in engine
Private Local_Toolbox As cToolbox                   ' Toolbox sub object
Private Local_Tileset As dxsTileset                 ' Tileset to use
Private Local_Map As dxsMap                         ' Map to use
Private Local_Displays As cDisplays                 ' Displays sub object
Private Local_Error As Boolean                      ' Error occured ?
Private Local_Surfaces As cSurfaces                 ' Surfaces sub object
Private Local_DirtySrc(9, 499) As RECT                 ' Dirty rectangle source
Private Local_DirtyDst(9, 499) As RECT              ' Dirty rectangle destination
Private Local_DirtyCount(9) As Integer              ' Counter for dirty rectangles

' CLASS PROPERTYS...

' Retrieve size of primary surface
Friend Property Get PrimaryWidth() As Integer
    PrimaryWidth = Local_PrimaryWidth
End Property
Friend Property Get PrimaryHeight() As Integer
    PrimaryHeight = Local_PrimaryHeight
End Property

' Retrieve map size
Friend Property Get MapTileHeight() As Integer
    MapTileHeight = Local_Map.TileHeight
End Property
Friend Property Get MapTileWidth() As Integer
    MapTileWidth = Local_Map.TileWidth
End Property
Friend Property Get MapUnitHeight() As Long
    MapUnitHeight = Local_Map.TileHeight * Local_Tileset.TileSize * 10
End Property
Friend Property Get MapUnitWidth() As Long
    MapUnitWidth = Local_Map.TileWidth * Local_Tileset.TileSize * 10
End Property

' Retrieve tile size
Friend Property Get TileSize() As Integer
    TileSize = Local_Tileset.TileSize
End Property

' Retrieve engine frame counter
Public Property Get FrameCount()
    FrameCount = Local_FrameCount
End Property

' Retrieve engine directx object
Public Property Get DDInstance() As DirectDraw2
    Set DDInstance = Local_DX
End Property

' Retrieve the last error
Public Property Get LastError() As String
    LastError = Local_LastError
End Property

' Retrieve engine error state
Public Property Get Error() As Boolean
    Error = Local_Error
    Local_Error = False
End Property


' Access to toolbox object
Public Property Set Toolbox(ByVal vData As Object)
    Set Local_Toolbox = vData
End Property
Public Property Get Toolbox() As cToolbox
    Set Toolbox = Local_Toolbox
End Property

' Access to displays object
Public Property Set Displays(ByVal vData As Object)
    Set Local_Displays = vData
End Property
Public Property Get Displays() As cDisplays
    Set Displays = Local_Displays
End Property

' Access to surfaces object
Public Property Set Surfaces(ByVal vData As Object)
    Set Local_Surfaces = vData
End Property
Public Property Get Surfaces() As cSurfaces
    Set Surfaces = Local_Surfaces
End Property


' CLASS FUNCTIONS AND PROCEDURES...

' AddDirtyRectangle: Adds a dirty rectangle to be redrawed
Friend Sub AddDirtyRectangle(ByVal nLeft As Integer, ByVal nTop As Integer, ByVal nWidth As Integer, ByVal nHeight As Integer, ByVal nDisplayIndex As Integer)

    Dim rSrc As RECT        ' Rectangle holding source area
    Dim rDst As RECT        ' Rectangle holding destination area
    Dim rBrd As RECT        ' Rectangle holding border for destination
    
    ' Calculate basic values
    With Displays.Item(nDisplayIndex)
    
        ' Calculate Border for destination (clipping)
        rBrd.Left = .PrimaryLeft
        rBrd.Right = rBrd.Left + .PrimaryWidth
        rBrd.Top = .PrimaryTop
        rBrd.Bottom = rBrd.Top + .PrimaryHeight
    
        ' Calculate Source rectangle
        rSrc.Left = nLeft
        rSrc.Top = nTop
        rSrc.Right = nLeft + nWidth
        rSrc.Bottom = nTop + nHeight
    
        ' Calculate Destination rectangle
        rDst.Left = nLeft + .PrimaryLeft - .PixelOffsetH
        rDst.Top = nTop + .PrimaryTop - .PixelOffsetV
        rDst.Right = rDst.Left + nWidth
        rDst.Bottom = rDst.Top + nHeight
        
    End With
    
    ' Calculate clipping
    If rDst.Left < rBrd.Left Then
        rSrc.Left = rSrc.Left + (rBrd.Left - rDst.Left)
        rDst.Left = rBrd.Left
    End If
    If rDst.Top < rBrd.Top Then
        rSrc.Top = rSrc.Top + (rBrd.Top - rDst.Top)
        rDst.Top = rBrd.Top
    End If
    If rDst.Right > rBrd.Right Then
        rSrc.Right = rSrc.Right - (rDst.Right - rBrd.Right)
        rDst.Right = rBrd.Right
    End If
    If rDst.Bottom > rBrd.Bottom Then
        rSrc.Bottom = rSrc.Bottom - (rDst.Bottom - rBrd.Bottom)
        rDst.Bottom = rBrd.Bottom
    End If

    ' Add dirty rectangle
    
    ' Source ...
    With Local_DirtySrc(nDisplayIndex, Local_DirtyCount(nDisplayIndex))
        .Left = rSrc.Left
        .Top = rSrc.Top
        .Right = rSrc.Right
        .Bottom = rSrc.Bottom
    End With
    
    ' Destination
    With Local_DirtyDst(nDisplayIndex, Local_DirtyCount(nDisplayIndex))
        .Left = rDst.Left
        .Top = rDst.Top
        .Right = rDst.Right
        .Bottom = rDst.Bottom
    End With

    ' Increase dirty rectangle counter if rectangle is valid
    With Local_DirtySrc(nDisplayIndex, Local_DirtyCount(nDisplayIndex))
        If .Bottom > .Top And .Right > .Left Then
            Local_DirtyCount(nDisplayIndex) = Local_DirtyCount(nDisplayIndex) + 1
        End If
    End With
    
End Sub

' ClearDirtyRectangleList: Clears all dirty rectangles
Friend Sub ClearDirtyRectangleList()

    Dim nRunDisplays As Integer     ' Variable to run through all displays
    
    ' Reset all dirty rectangle counters
    For nRunDisplays = 0 To 9
        Local_DirtyCount(nRunDisplays) = 0
    Next
    
End Sub
' Initialize: create subordinate objects
Private Sub Class_Initialize()

    ' TEMPORARY INSERTED FOR DEBUGGING ... REMOVE IN RUNTIME
    DebugInit
    ' ==================================================================
    
    Set Local_Toolbox = New cToolbox
    Set Local_Toolbox.Parent = Me
    
    Set Local_Displays = New cDisplays
    Set Local_Displays.Parent = Me
    
    Set Local_Surfaces = New cSurfaces
    Set Local_Surfaces.Parent = Me
    
End Sub

' Terminate: Drop sub objects
Private Sub Class_Terminate()

    Set Local_Toolbox = Nothing
    Set Local_Displays = Nothing
    Set Local_Surfaces = Nothing
    
End Sub


' GenerateError Method: Sets the engine error variables for retrieval by user
Friend Sub GenerateError(nError As Integer)

    Dim eNumber As String, eModule As String, eText As String
    
    eNumber = Format(nError, "00000")
    
    Select Case nError \ 100
        
        Case 1
            eModule = "engine.boot"
        Case 2
            eModule = "engine.shutdown"
        Case 3
            eModule = "engine.redraw"
        Case 4
            eModule = "engine.recalc"
        Case 5
            eModule = "engine.refresh"
        Case 6
            eModule = "engine.toolbox.createsurface"
        Case 7
            eModule = "engine.toolbox.loadbitmap"
        Case 8
            eModule = "engine.loadtileset"
        Case 9
            eModule = "engine.loadmap"
        Case 10
            eModule = "engine.displays.item().initialize"
        Case 11
            eModule = "engine.displays.add"
        Case 12
            eModule = "engine.redrawdisplays"
        Case 13
            eModule = "engine.displays.item().scroll"
        Case 14
            eModule = "engine.recalcanisquares"
        Case 15
            eModule = "engine.redrawanisquare"
        Case 16
            eModule = "engine.recalcanisquarelist"
        Case 17
            eModule = "engine.displays.item().locksurface"
        Case 18
            eModule = "engine.displays.item().unlocksurface"
        Case 19
            eModule = "engine.surfaces.item().loadfromfile"
        Case 20
            eModule = "engine.surfaces.item().render"
        Case 21
            eModule = "engine.surfaces.add"
            
        Case Else
            eModule = "Unknown"
            
    End Select
    
    If nError Mod 100 = 0 Then
        eText = "General error"
    Else
    
        Select Case nError
            
            Case 101
                eText = "Invalid window handle passed"
            Case 102
                eText = "Unable to create DirectDraw instance"
            Case 103
                eText = "Invalid resolution passed"
            Case 104
                eText = "Unable to create primary surface"
            Case 105
                eText = "Unable to create backbuffer surface"
                    
            Case 601
                eText = "Surface creation failed"
                    
            Case 701
                eText = "Bitmap file could not be opened"
            Case 702
                eText = "Bitmap load failed"
            Case 703
                eText = "Surface creation failed"
            Case 704
                eText = "Memory DC creation failed"
                    
            Case 801
                eText = "Bitmap file could not be opened"
            Case 802
                eText = "Definition file could not be opened"
            Case 803
                eText = "Tileset surface creation failed"
                
            Case 901
                eText = "Map file could not be opened"
                
            Case 1001
                eText = "Invalid display area passed"
            Case 1002
                eText = "Display surface creation failed"
            Case 1003
                eText = "Display area to small for map viewport"
            Case 1004
                eText = "Display width or height does not equal multiple tilesize"
                
            Case 1301
                eText = "No tileset loaded"
                
            Case 1701
                eText = "Device context creation failed"
                
            Case 1801
                eText = "Display is not locked"
            
            Case 1901
                eText = "Surface could not be loaded"
                
            Case 2001
                eText = "Invalid destination surface"
                
            Case 2101
                eText = "Surface load on addnew failed"
                
            Case Else
                eText = "Unknown error"
                

                
        End Select
    
    End If
    
    Local_LastError = Local_LastError + eNumber + " [" + UCase(eModule) + "] " + eText + IIf(Err.Number <> 0, " (" & Err.Number & " " & Err.Description & ")", "") + vbCrLf
    Local_Error = True
        
End Sub


' Boot method: Initializes DirectX
Public Function Boot(hWndDisplay As Long, Resolution As dxsResolution) As Boolean

     ' Enable error handling
     On Error GoTo E_Boot
     Boot = False
     
     Dim tempDXD As DDSURFACEDESC    ' Localc surface description
     Dim tempDXC As DDSCAPS          ' Local display capabilities description
    
     ' Check validity of window handle
     If hWndDisplay < 1 Then
        GenerateError 101
        Exit Function
     End If
     
     ' Store window handle locally (for shutdown method)
     Local_hWndDisplay = hWndDisplay
     
     ' Create DX instance
     DirectDrawCreate ByVal 0&, Local_DX, Nothing
     
     ' Check validity of instance
     If Local_DX Is Nothing Then
        GenerateError 102
     End If
     
     ' Set cooperation mode
     Local_DX.SetCooperativeLevel hWndDisplay, DDSCL_EXCLUSIVE Or DDSCL_FULLSCREEN
     
     ' Set display mode ...
     Select Case Resolution
     
        ' LoRes
        Case dxsResLo
            Local_DX.SetDisplayMode 640, 480, 16, 0, 0
            Local_PrimaryWidth = 640
            Local_PrimaryHeight = 480
            
        ' HiRes
        Case dxsResHi
            Local_DX.SetDisplayMode 800, 600, 16, 0, 0
            Local_PrimaryWidth = 800
            Local_PrimaryHeight = 600
            
        ' Invalid
        Case Else
            GenerateError 103
            
     End Select
     
     ' Initialize front buffer description ...
     With tempDXD
         ' Get Structure size
         .dwSize = Len(tempDXD)
         ' Structure uses Surface Caps and count of BackBuffers
         .dwFlags = DDSD_CAPS Or DDSD_BACKBUFFERCOUNT
         ' Structure describes a flippable (buffered) surface
         .DDSCAPS.dwCaps = DDSCAPS_PRIMARYSURFACE Or DDSCAPS_FLIP Or DDSCAPS_COMPLEX Or DDSCAPS_SYSTEMMEMORY
         ' Structure uses one BackBuffer
         .dwBackBufferCount = 1
     End With
     
     ' Create front buffer from structure
     Local_DX.CreateSurface tempDXD, Local_DXSPrimary, Nothing
     
     ' Check frontbuffer existance
     If Local_DXSPrimary Is Nothing Then
        GenerateError 104
     End If
     
     ' Create back buffer from front buffer
     tempDXC.dwCaps = DDSCAPS_BACKBUFFER
     Local_DXSPrimary.GetAttachedSurface tempDXC, Local_DXSBackbuffer
     
     ' Check backbuffer existance
     If Local_DXSBackbuffer Is Nothing Then
        GenerateError 105
     End If
     
     ' Leave boot function with success
     Boot = True
     Exit Function

E_Boot:
    
    ' Error handling
    GenerateError 100
    
End Function

' Shutdown method: Frees all DirectX ressources, unlocks window
Public Sub Shutdown()
   
   ' Enable error handling
   On Error GoTo E_Shutdown
   
   ' Flip from DX-Surface to standard GDI
    Local_DX.FlipToGDISurface
    
    ' Restore old resolution and depth
    Local_DX.RestoreDisplayMode
    
    ' Return control to windows
    Local_DX.SetCooperativeLevel Local_hWndDisplay, DDSCL_NORMAL
    
    ' !IMPORTANT! Clear all DX Objects
    If Not Local_DXSBackbuffer Is Nothing Then Set Local_DXSBackbuffer = Nothing
    If Not Local_DXSPrimary Is Nothing Then Set Local_DXSPrimary = Nothing
    If Not Local_Tileset.ddSurface Is Nothing Then Set Local_Tileset.ddSurface = Nothing
    
    Exit Sub
    
E_Shutdown:

    GenerateError 200
    
End Sub

' Redraw method: Refreshes all graphixs, flips surfaces
Public Function Redraw() As Boolean

    ' Enable error handling
    On Error GoTo E_Redraw
    Redraw = False
    
    ' Redraw all displays
    Call RedrawDisplays
    
    ' Flip backbuffer to screen
    Call FlipBuffers
    
    ' Increase internal frame counter
    Local_FrameCount = Local_FrameCount + 1
    
    Redraw = True
    Exit Function

E_Redraw:
    
    GenerateError 300

End Function

' Recalc method: Recalculates all variables, but does not draw
Public Function Recalc() As Boolean

    ' Enable error handling
    On Error GoTo E_Recalc
    Recalc = False
    
    ' Recalculate animated mapsquares
    RecalcAniSquares
    
    Recalc = True
    Exit Function
    
E_Recalc:

    GenerateError 400
    
End Function

' Refresh method: Does recalc, then refresh
Public Function Refresh() As Boolean

    On Error GoTo E_Refresh
    Refresh = False
    
    Recalc
    Redraw
    
    Refresh = True
    Exit Function
    
E_Refresh:

    GenerateError 500
    
End Function

' FlipBuffers method: Flips the backbuffer and the frontbuffer
Private Sub FlipBuffers()

    ' Enforce proceed after error in case of lost surface
    On Error Resume Next
    
    ' Try to flip until success
    Do
    
        ' Try to flip
        Local_DXSPrimary.Flip Nothing, 0
        
        ' Restore front buffer if flipping fails
        If Err.Number = DDERR_SURFACELOST Then Local_DXSPrimary.Restore
        
    Loop Until Err.Number = 0

    On Error GoTo 0

End Sub

' LoadTileset method: Loads a tileset matching the engine tileset format
Public Function LoadTileset(FileName As String) As Boolean

    Dim nRunTiles As Integer        ' Variable to run though all tiles found
    Dim nTileCount As Integer       ' Number of tiles to be loaded
    Dim nTileSize As Integer        ' Input tile size
    
    ' Temporary Tile description variables for current tile to load from file
    Dim nTop As Integer
    Dim nLeft As Integer
    Dim nAnimationFrames As Byte
    Dim nAltitude As Byte
    Dim nAnimationDelay As Integer
    Dim nAnimationRandom As Integer
    Dim nColor As Long

    ' Enable error handling
    On Error GoTo E_LoadTileset
    LoadTileset = False
    
    ' Check for bitmap file existance
    On Error GoTo E_LoadTileset_NoBitmapFile
    Open FileName + ".bmp" For Input As #1
    Close #1
    On Error GoTo E_LoadTileset

    ' Check for definition file existance
    On Error GoTo E_LoadTileset_NoDefinitionFile
    Open FileName + ".def" For Input As #1
    Close #1
    On Error GoTo E_LoadTileset
    
    ' Open tileset definition
    Open FileName + ".def" For Input As #1

    ' Read tile size
    Input #1, Local_Tileset.TileSize
    
    ' Read number of tiles to be processed
    Input #1, nTileCount
    
    ' Read all the tile definitions
    For nRunTiles = 0 To nTileCount - 1
        With Local_Tileset.Tiles(nRunTiles)
        Input #1, .Left, .Top, .AnimationFrames, .Altitude, .AnimationDelay, .AnimationRandom, .Color
        End With
    Next
    
    ' Close tileset definition
    Close #1

    ' Clear DirectX Surface holding tileset if already loaded
    If Not Local_Tileset.ddSurface Is Nothing Then
        Set Local_Tileset.ddSurface = Nothing
    End If
    
    ' Load Tilesource bitmap into Tileset surface
    Set Local_Tileset.ddSurface = Local_Toolbox.LoadBitmap(FileName + ".BMP")
    
    ' Check for success of surface load
    If Local_Tileset.ddSurface Is Nothing Then
        GenerateError 803
        Exit Function
    End If
    
    LoadTileset = True
    Exit Function
    
E_LoadTileset:
 
    Close #1
    GenerateError 800
    Exit Function
    
E_LoadTileset_NoBitmapFile:

    Close #1
    GenerateError 801
    Exit Function

E_LoadTileset_NoDefinitionFile:

    Close #1
    GenerateError 802
    Exit Function

End Function

' LoadMap method: Loads a map file matching the engine map format
Public Function LoadMap(ByVal FileName As String) As Boolean

    Dim nRunX As Integer        ' Variable to run though all squares
    Dim nRunY As Integer        ' Variable to run though all squares
    Dim nMaxX As Integer        ' Variable holding x size of map
    Dim nMaxY As Integer        ' Variable holding y size of map
    Dim sLine As String         ' Square parameters for the current square to load
    
    ' Enable error handling
    On Error GoTo E_LoadMap
    LoadMap = False
    
    ' Check for bitmap file existance
    On Error GoTo E_LoadMap_NoFile
    Open FileName + ".map" For Input As #1
    Close #1
    On Error GoTo E_LoadMap
    
    ' Open map file
    Open FileName + ".map" For Input As #1

    ' Skip irrelevant information
    Input #1, sLine
    Input #1, sLine
    
    ' Input map size, dimension square array
    Input #1, nMaxX, nMaxY
    ReDim Local_Map.Square(nMaxX, nMaxY)
    
    ' Set map size
    Local_Map.TileWidth = nMaxX
    Local_Map.TileHeight = nMaxY
    
    ' Read all the squares
    For nRunY = 0 To nMaxY - 1
        Input #1, sLine
        For nRunX = 0 To nMaxX - 1
            Local_Map.Square(nRunX, nRunY).TileIndex = Val(Mid(sLine, nRunX * 3 + 1, 3))
            Local_Map.Square(nRunX, nRunY).AnimationFrame = 0
        Next
    Next
    
    ' Close map definition
    Close #1
    
    ' Return success
    LoadMap = True
    Exit Function
    
E_LoadMap:
 
    Close #1
    GenerateError 900
    Exit Function
    
E_LoadMap_NoFile:

    Close #1
    GenerateError 901
    
End Function

' Redraw: Redraws the displays according to current settings for redraw operations
Friend Function RedrawDisplays() As Boolean

    Dim nRunDisplays As Integer         ' Variable to run through displays
    
    ' Variables used for total redraw of map viewport display
    Dim nRunMapH As Integer, nRunMapV As Integer
    Dim rSource As RECT
    Dim nDestH As Integer, nDestV As Integer
    Dim nTileIndex As Integer
    Dim nAniLookupIndex As Integer
    Dim nRunDirty As Integer
    Dim nListCount As Integer
    
    ' Variables used for full redraw of any display
    Dim nOffsetH As Integer, nOffsetV As Integer
    
    ' Enable error handling
    On Error GoTo E_RedrawDisplays
    RedrawDisplays = False
    
    ' Run through all displays available ...
    For nRunDisplays = 1 To Displays.Count
        With Displays.Item(nRunDisplays)
            
            ' Exit if no redraw is necessary
            If Not .RedrawState = dxsRSNone Then
            
                ' Redraw depending on display type...
                
                ' Viewport ...
                If .DisplayType = dxsDTViewport Then
                    
                    ' Redraw depending on redraw state...
                    
                    ' Total background redraw: Bilt all tiles to display buffer
                    If .RedrawState = dxsRSBackgroundTotal Then

                        ' Run through the visible map squares
                        For nRunMapH = 0 To .MapWidth
                            For nRunMapV = 0 To .MapHeight
                                
                                ' Calculate current tile index
                                nTileIndex = Local_Map.Square(nRunMapH + .MapPosH, nRunMapV + .MapPosV).TileIndex
                                
                                ' Calculate source position
                                If Local_Map.Square(nRunMapH + .MapPosH, nRunMapV + .MapPosV).AnimationFrame > 0 Then
                                    rSource.Left = Local_Tileset.Tiles(nTileIndex).Left + Local_Map.Square(nRunMapH + .MapPosH, nRunMapV + .MapPosV).AnimationFrame * Local_Tileset.TileSize
                                Else
                                    rSource.Left = Local_Tileset.Tiles(nTileIndex).Left
                                End If
                                rSource.Top = Local_Tileset.Tiles(nTileIndex).Top
                                rSource.Right = rSource.Left + Local_Tileset.TileSize
                                rSource.Bottom = rSource.Top + Local_Tileset.TileSize
                                
                                ' Calculate destination within display buffer
                                nDestH = nRunMapH * Local_Tileset.TileSize
                                nDestV = nRunMapV * Local_Tileset.TileSize
                                
                                ' Blit to display backbuffer
                                .ddSurface.BltFast nDestH, nDestV, Local_Tileset.ddSurface, rSource, DDBLTFAST_NOCOLORKEY Or DDBLTFAST_WAIT
                                
                            Next
                        Next
                    
                    End If
                    
                    ' Full backbuffer redraw: Blit from displaybuffer to background
                    If .RedrawState = dxsRSBackbufferTotal Or .RedrawState = dxsRSBackgroundTotal Then
                        
                        ' Calculate source position
                        rSource.Left = .PixelOffsetH
                        rSource.Top = .PixelOffsetV
                        rSource.Right = rSource.Left + .PrimaryWidth
                        rSource.Bottom = rSource.Top + .PrimaryHeight
                        
                        ' Blit to backbuffer
                        Local_DXSBackbuffer.BltFast .PrimaryLeft, .PrimaryTop, .ddSurface, rSource, DDBLTFAST_NOCOLORKEY Or DDBLTFAST_WAIT
                    
                    End If
                    
                    ' Partial backbuffer redraw ... Dirty rectangle redraw: Blit dirty rectangle list from displaybuffer to background
                    If .RedrawState = dxsRSBackbufferpartial Then
                    
                        ' Run through all dirty rectangles of the display
                        For nRunDirty = 0 To Local_DirtyCount(nRunDisplays) - 1
        
                            ' Calculate source position
                            rSource.Left = Local_DirtySrc(nRunDisplays, nRunDirty).Left
                            rSource.Top = Local_DirtySrc(nRunDisplays, nRunDirty).Top
                            rSource.Right = Local_DirtySrc(nRunDisplays, nRunDirty).Right
                            rSource.Bottom = Local_DirtySrc(nRunDisplays, nRunDirty).Bottom
                            
                            ' Blit to backbuffer
                            Local_DXSBackbuffer.BltFast Local_DirtyDst(nRunDisplays, nRunDirty).Left, Local_DirtyDst(nRunDisplays, nRunDirty).Top, .ddSurface, rSource, DDBLTFAST_NOCOLORKEY Or DDBLTFAST_WAIT
                        
                        Next
                        
                    End If
                    
                    ' Reset redraw state
                    If Not .RedrawState = dxsRSNone Then .RedrawState = dxsRSBackbufferpartial
                    
                End If
                
                ' Radar ...
                If .DisplayType = dxsDTRadar Then
                    
                    ' Draw radar map
                    If .RedrawState <> dxsRSNone Then
                    
                        Dim nHDC As Long        ' Variable to hold DC for drawing
                        Dim DXFX As DDBLTFX
                        
                        ' Clear radar surface
                        With DXFX
                            .dwSize = Len(DXFX)
                            .dwFillColor = RGB(0, 0, 0)
                        End With
                        rSource.Top = 0
                        rSource.Left = 0
                        rSource.Bottom = .PrimaryWidth
                        rSource.Right = .PrimaryHeight
                        .ddSurface.Blt rSource, Nothing, rSource, DDBLT_COLORFILL, DXFX
                        
                        ' Get DC on ddsurface
                        .ddSurface.Restore
                        .ddSurface.GetDC nHDC

                        ' Draw radar map
                        For nRunMapH = 0 To .PrimaryWidth
                            If nRunMapH + .MapPosH < Local_Map.TileWidth Then
                                For nRunMapV = 0 To .PrimaryHeight
                                    If nRunMapV + .MapPosV < Local_Map.TileHeight Then
                                        SetPixelV nHDC, nRunMapH, nRunMapV, Local_Tileset.Tiles(Local_Map.Square(nRunMapH + .MapPosH, nRunMapV + .MapPosV).TileIndex).Color
                                    End If
                                Next
                            End If
                        Next
                        
                        ' Free DC
                        .ddSurface.ReleaseDC nHDC
                        
                        ' Blit radar map to backbuffer ...
                        
                        ' Calculate source position
                        rSource.Left = 0
                        rSource.Top = 0
                        rSource.Right = .PrimaryWidth
                        rSource.Bottom = .PrimaryHeight
                        
                        ' Blit to backbuffer
                        Local_DXSBackbuffer.BltFast .PrimaryLeft, .PrimaryTop, .ddSurface, rSource, DDBLTFAST_NOCOLORKEY Or DDBLTFAST_WAIT

                    End If
                    
                    ' Reset redraw state
                    .RedrawState = dxsRSNone
                    
                End If
                
                ' Panel ...
                If .DisplayType = dxsDTPanel Then

                    ' Full backbuffer redraw: Blit from displaybuffer to background
                    If .RedrawState <> dxsRSNone Then
                        
                        ' Calculate source position
                        rSource.Left = 0
                        rSource.Top = 0
                        rSource.Right = .PrimaryWidth
                        rSource.Bottom = .PrimaryHeight
                        
                        ' Blit to backbuffer
                        Local_DXSBackbuffer.BltFast .PrimaryLeft, .PrimaryTop, .ddSurface, rSource, DDBLTFAST_NOCOLORKEY Or DDBLTFAST_WAIT
                    
                    End If
                    
                    ' Reset redraw state
                    .RedrawState = dxsRSNone

                End If
                
            End If
            
        End With
        
    Next
    
    ' Clear dirty rectangle list
    Call ClearDirtyRectangleList
        
    RedrawDisplays = True
    Exit Function
    
E_RedrawDisplays:

    GenerateError 1200
    
End Function

' RecalcAniSquareList: Searches all displays for animated squares, adds them to anisquare collection
Friend Function RecalcAniSquareList() As Boolean

    Dim nRunDisplays As Integer                     ' Variable to run through all display
    Dim nRunMapH As Integer, nRunMapV As Integer    ' Variable to run through viewport map tiles
    
    ' Enable error handling
    On Error GoTo E_RecalcAniSquareList
    RecalcAniSquareList = False
    
    ' Clear ani square collection
    Displays.Anisquares.Clear
    
    ' Run through all displays
    For nRunDisplays = 1 To Displays.Count
        With Displays.Item(nRunDisplays)
        
            ' Only calculate for viewport displays
            If .DisplayType = dxsDTViewport Then
            
                ' Run through tiles within display
                For nRunMapH = .MapPosH To .MapPosH + .MapWidth
                    For nRunMapV = .MapPosV To .MapPosV + .MapHeight
                        ' If square contains anitile, add it to anisquare collection
                        If Local_Tileset.Tiles(Local_Map.Square(nRunMapH, nRunMapV).TileIndex).AnimationFrames > 0 Then
                            Displays.Anisquares.Add nRunMapH, nRunMapV
                        End If
                    Next
                Next
            End If
            
        End With
    Next
    
    RecalcAniSquareList = True
    Exit Function
    
E_RecalcAniSquareList:

    GenerateError 1600
    
End Function

' Redrawdisplaysquare: Redraws a single map square within a viewport display
Friend Function RedrawAniSquare(ByVal nPosH As Long, ByVal nPosV As Long, ByVal nDisplayIndex As Long) As Boolean

    Dim nTileIndex As Integer                   ' Variable to hold index of tile within tileset
    Dim rSource As RECT                         ' Rectangle type to hold blit source position
    Dim nDestH As Integer, nDestV As Integer    ' Target coordinates of blit
    
    ' Enable error handling
    On Error GoTo E_RedrawAniSquare
    RedrawAniSquare = False
    
    ' Calculate current tile index
    nTileIndex = Local_Map.Square(nPosH, nPosV).TileIndex
    
    ' Calculate source position
    If Local_Map.Square(nPosH, nPosV).AnimationFrame > 0 Then
        rSource.Left = Local_Tileset.Tiles(nTileIndex).Left + Local_Map.Square(nPosH, nPosV).AnimationFrame * Local_Tileset.TileSize
    Else
        rSource.Left = Local_Tileset.Tiles(nTileIndex).Left
    End If
    rSource.Top = Local_Tileset.Tiles(nTileIndex).Top
    rSource.Right = rSource.Left + Local_Tileset.TileSize
    rSource.Bottom = rSource.Top + Local_Tileset.TileSize
    
    ' Calculate destination within display buffer
    nDestH = (nPosH - Displays.Item(nDisplayIndex).MapPosH) * Local_Tileset.TileSize
    nDestV = (nPosV - Displays.Item(nDisplayIndex).MapPosV) * Local_Tileset.TileSize
    
    ' Blit to display backbuffer
    Displays.Item(nDisplayIndex).ddSurface.BltFast nDestH, nDestV, Local_Tileset.ddSurface, rSource, DDBLTFAST_NOCOLORKEY Or DDBLTFAST_WAIT

    RedrawAniSquare = True
    Exit Function
    
E_RedrawAniSquare:

    GenerateError 1500
    
End Function

' RecalcAniSquares: Run through all animated squares and calulate their animations
Friend Function RecalcAniSquares() As Boolean

    Dim nRunDisplays As Integer                     ' Variable to run through displays
    Dim nRunAniSquares As Integer                   ' Variable to run through animated tiles
    Dim nSquareH As Integer, nSquareV As Integer    ' Position of animated tile
    Dim bRedrawSquare As Boolean
    
    ' Enable error handling
    On Error GoTo E_RecalcAniSquares
    RecalcAniSquares = False
    
    ' Run through all entrys in lookup list
    For nRunAniSquares = 0 To Displays.Anisquares.Count - 1
        
        ' Set redraw necessary to false
        bRedrawSquare = False
        
        ' Retrieve coordinates of animated square
        nSquareH = Displays.Anisquares.Item(nRunAniSquares).PositionH
        nSquareV = Displays.Anisquares.Item(nRunAniSquares).PositionV
        With Local_Map.Square(nSquareH, nSquareV)
        
            ' Animate tile, if animated
            If .AnimationFrame > 0 Then
            
                ' If frame advance necessary ...
                If Local_FrameCount >= .AnimationAdvance Then
                
                    ' Advance frame ...
                    .AnimationFrame = Local_Map.Square(nSquareH, nSquareV).AnimationFrame + 1
                    .AnimationAdvance = Local_FrameCount + Local_Tileset.Tiles(.TileIndex).AnimationDelay
                    
                    ' If sequence end reached, stop animation
                    If .AnimationFrame >= Local_Tileset.Tiles(.TileIndex).AnimationFrames Then
                        .AnimationFrame = 0
                    End If
                    
                    ' Redraw necessary
                    bRedrawSquare = True
                    
                End If
                
            ' Randomize animation start, if not animated
            Else
                
                If Int(Rnd(1) * Local_Tileset.Tiles(.TileIndex).AnimationRandom) = 1 Then

                    ' Start animation
                    .AnimationFrame = 1
                    .AnimationAdvance = Local_FrameCount + Local_Tileset.Tiles(.TileIndex).AnimationDelay
                    
                    ' Redraw necessary
                    bRedrawSquare = True
                    
                End If
            
            End If
            
        End With
        
        ' Calculate redraw operations for displays...
        If bRedrawSquare Then
        
            ' Run through displays ...
            For nRunDisplays = 1 To Displays.Count
                With Displays.Item(nRunDisplays)
                
                    ' Only calculate for viewport displays
                    If .DisplayType = dxsDTViewport Then
                        
                        ' If square is visible in viewport, redraw
                        If .MapPosH <= nSquareH Then
                            If .MapPosV <= nSquareV Then
                                If .MapPosH + .MapWidth >= nSquareH Then
                                    If .MapPosV + .MapHeight >= nSquareV Then
                                        If .RedrawState <> dxsRSBackgroundTotal Then
                                            RedrawAniSquare nSquareH, nSquareV, nRunDisplays
                                            AddDirtyRectangle (nSquareH - .MapPosH) * Local_Tileset.TileSize, (nSquareV - .MapPosV) * Local_Tileset.TileSize, Local_Tileset.TileSize, Local_Tileset.TileSize, nRunDisplays
                                        End If
                                    End If
                                End If
                            End If
                        End If
                        
                    End If
                    
                End With
            Next
            
        End If
        
    Next
    
    RecalcAniSquares = True
    Exit Function
    
E_RecalcAniSquares:

    GenerateError 1400
    
End Function
