Attribute VB_Name = "mSample"
Option Explicit

' API DECLARATIONS

Declare Function SetPixelV Lib "gdi32" (ByVal hdc As Long, ByVal X As Long, ByVal Y As Long, ByVal crColor As Long) As Long
Declare Function CreateSolidBrush Lib "gdi32" (ByVal crColor As Long) As Long
Declare Function Rectangle Lib "gdi32" (ByVal hdc As Long, ByVal X1 As Long, ByVal Y1 As Long, ByVal X2 As Long, ByVal Y2 As Long) As Long
Declare Function SelectObject Lib "gdi32" (ByVal hdc As Long, ByVal hObject As Long) As Long
Declare Function DeleteObject Lib "gdi32" (ByVal hObject As Long) As Long


' APPLICATION-SPECIFIC DEFINITIONS

' Program Status
Type tProgram
    Initialized As Boolean          ' Already initialized ?
    Running As Boolean              ' Already running (or terminated?)
    Title As String                 ' Display in startup window
    MouseX As Integer               ' Coordinates of mouse
    MouseY As Integer               ' Coordinates of mouse
    ScrollSpeed As Integer          ' SpeedOMeter for scrolling main viewport
    OverFlowCount As Integer         ' Counter of frames that took to much time
End Type
Public Program As tProgram

' DXS Engine
Public MyEngine As Engine

' Initialize engine and application
Sub AppInit()
    
    ' Enable error handling: In any DirectX app, it is very important to
    ' return control to windows after an error occured !
    On Error GoTo E_AppInit
    
    ' Initialize application data
    With Program
        .Initialized = True
        .Title = "The Engine"
        .ScrollSpeed = 10
    End With
    
    ' Display title dialog
    fStartup.Show 1
    
    ' Start the engine ...
    
    ' Create an instance of the engine
    Set MyEngine = New Engine
    
    ' Start engine, provide handle of window to use and resolution
     MyEngine.Boot fSample.hWnd, dxsResLo
    
    ' Check if error on boot occured, leave program if boot failed
    If MyEngine.Error Then
        Debug.Print MyEngine.LastError
        Call AppExit
    End If
    
    ' Load a tileset definition and bitmap file into the engine
    MyEngine.LoadTileset App.Path + "\..\common files\default"
    
    ' Load a map file into the engine
    MyEngine.LoadMap App.Path + "\..\common files\default"
    
    ' Add main display
    MyEngine.Displays.Add 10, 10, 400, 400, dxsDTViewport, "MAIN"
    
    ' Add camera display #1
    MyEngine.Displays.Add 420, 10, 200, 200, dxsDTViewport, "VIDEO1"
    
    ' Add camera display #2
    MyEngine.Displays.Add 420, 210, 200, 200, dxsDTViewport, "VIDEO2"
    
    ' Add overview display
    MyEngine.Displays.Add 10, 420, 50, 50, dxsDTRadar, "OVERVIEW"
    
    ' Add panel display
    MyEngine.Displays.Add 70, 420, 550, 50, dxsDTPanel, "CONTROL"
    
    ' Load panel bitmap into new surface
    MyEngine.Surfaces.Add App.Path + "\panel.bmp", "PANEL"
    
    ' Render loaded surface onto display
    MyEngine.Surfaces.Item("PANEL").Render MyEngine.Displays.Item("CONTROL").ddSurface, 0, 0, 550, 50, 0, 0
    
    ' Enforce redraw of display
    MyEngine.Displays.Item("CONTROL").EnforceRedraw
    
    ' Scroll the displays to the initial position
    MyEngine.Displays.Item("MAIN").Scroll 1000, 1000, dxsAbsolute
    MyEngine.Displays.Item("VIDEO1").Scroll 1000, 1000, dxsAbsolute
    MyEngine.Displays.Item("VIDEO2").Scroll 1000, 1000, dxsAbsolute
    MyEngine.Displays.Item("OVERVIEW").Scroll 1000, 1000, dxsAbsolute
    
    ' Check for any error during initialisation, exit application if error occured
    If MyEngine.Error Then
        Debug.Print Err.Description
        Debug.Print MyEngine.LastError
        Call AppExit
    End If
    
    Exit Sub
    
E_AppInit:

    ' Error handling code: Write engine error, return control to windows
    Debug.Print Err.Description
    If MyEngine.Error Then
        Debug.Print MyEngine.LastError
    End If
    Call AppExit
    
End Sub

' Unload application data
Sub AppExit()

    ' Shut down engine
    If Not MyEngine Is Nothing Then MyEngine.Shutdown
    Set MyEngine = Nothing
    
    ' End sample app
    Unload fSample
    End
        
End Sub

' Main Loop of the application
Sub AppLoop()

    ' Enable error handling: In any DirectX app, it is very important to
    ' return control to windows after an error occured !
    On Error GoTo E_AppLoop
    
    Dim nNextFrameAdvance As Double     ' Variable to hold next frame timing
    
    ' Note: This loop is a standard construction, you should use a similiar loop in your app!

    ' main loop started - set status
    Program.Running = True
    
    Do
    
        ' Start frame timing
        nNextFrameAdvance = Timer + 0.02
        
        
        ' Refresh graphics
        MyEngine.Refresh
                
        ' Check for any error produced by the engine
        If MyEngine.Error Then
            Debug.Print MyEngine.LastError
            Call AppExit
        End If
        
        'Receive user input
        DoEvents
        
        ' Do application specific code
        Call AppSpecific
        
        ' Display frametime overflow ...
        If Not (Timer < nNextFrameAdvance) Then
        
            Program.OverFlowCount = Program.OverFlowCount + 1
            
        ' Display frametime okay ...
        Else
            ' Delay until frametime reached
            Do
            Loop While Timer < nNextFrameAdvance
        End If
        
    Loop While Program.Running
    
    ' Clean up application, stop sample app
    Call AppExit

E_AppLoop:

    ' Error handling code: Write engine error, return control to windows
    If MyEngine.Error Then
        Debug.Print MyEngine.LastError
    End If
    Call AppExit

End Sub

Sub AppSpecific()

    ' Enable error handling: In any DirectX app, it is very important to
    ' return control to windows after an error occured !
    On Error GoTo Err_AppSpecific
    
    
    ' Update the panel display every second, using standard API dc functions
    ' In this case, draw the frame time overflows as bar graph
    If MyEngine.FrameCount Mod 50 = 0 Then
        Debug.Print MyEngine.FrameCount
        Debug.Print Program.OverFlowCount
        Dim nHDC As Long        ' Variable to hold acquired device context
        Dim nBrushBack As Long  ' Black brush for clearing the background
        Dim nBrushGreen As Long ' Green brush for drawing the bar
        Dim nPercentVal As Integer  ' Value for bar graph calculation
        
        ' Acquire device context for panel display
        nHDC = MyEngine.Displays.Item("CONTROL").LockSurface
        
        ' Acquisition succeeded ... Do anything you want like with a standard dc
        If nHDC > 0 Then

            ' In this case ... draw the bargraph ...
            nPercentVal = IIf(Program.OverFlowCount \ 2 > 25, 25, Program.OverFlowCount \ 2)
            nBrushBack = CreateSolidBrush(RGB(0, 0, 0))
            nBrushGreen = CreateSolidBrush(RGB(64, 192, 64))
            SelectObject nHDC, nBrushBack
            Rectangle nHDC, 525, 12, 535, 37
            SelectObject nHDC, nBrushGreen
            Rectangle nHDC, 525, 37 - (nPercentVal), 535, 37
            DeleteObject nBrushGreen
            DeleteObject nBrushBack
            
            ' Reset overflow counter
            Program.OverFlowCount = 0
            
        End If

        ' Free and update device context, redrawing is automatically enforced
        MyEngine.Displays.Item("CONTROL").UnlockSurface

        ' Reset frame time overflow counter
        Program.OverFlowCount = 0
        
    End If
    
    
    ' Scroll the map according to mouse, using Engine functions ...
    
    Dim nScrollX As Long, nScrollY As Long        ' Variables holding current scroll values
    
    ' Set scroll offset according to mouse position variables stored in program typevar
    If Program.MouseX < 5 Then
        nScrollX = -Program.ScrollSpeed
    End If
    If Program.MouseX > 635 Then
        nScrollX = Program.ScrollSpeed
    End If
    If Program.MouseY < 5 Then
        nScrollY = -Program.ScrollSpeed
    End If
    If Program.MouseY > 475 Then
        nScrollY = Program.ScrollSpeed
    End If
    
    ' Scroll da display
    If nScrollX <> 0 Or nScrollY <> 0 Then MyEngine.Displays.Item("MAIN").Scroll nScrollX, nScrollY, dxsRelative
    
    Exit Sub
    
Err_AppSpecific:
    
    ' Error handling code: Write engine error, return control to windows
    If MyEngine.Error Then
        Debug.Print MyEngine.LastError
    End If
    Call AppExit
    
End Sub
