Attribute VB_Name = "mEditor"
' --------------------------------------------------------------------------------------
' THIS MODULE CONTAINS CODE SPECIFIC FOR THE  MAP Editor.
' REPLACE AT WILL, BUT NOTE: THE ORIGINAL DIRECTXSUITE APPLICATION MAY NO LONGER
' WORK WITH MAPS GENERATED USING YOUR MODIFIED EDITOR.
' --------------------------------------------------------------------------------------

' NOTE: This App does not use dirty rectangles or any of the
'       fancy tricks a real game engine needs. It's just a dumb editor.
'       For the real cool stuff, check out the engine sample.

Option Explicit


' API DECLARATIONS

' Constants and types for use with win32 API
Const IMAGE_BITMAP = 0
Const LR_LOADFROMFILE = &H10
Const SRCCOPY = &HCC0020

' Window declarations
Declare Function SetWindowPos Lib "user32" (ByVal hwnd As Long, ByVal hWndInsertAfter As Long, ByVal X As Long, ByVal Y As Long, ByVal cx As Long, ByVal cy As Long, ByVal wFlags As Long) As Long

' Single Pixel manipulation
Declare Function GetPixel Lib "gdi32" (ByVal hdc As Long, ByVal X As Long, ByVal Y As Long) As Long
Declare Function SetPixelV Lib "gdi32" (ByVal hdc As Long, ByVal X As Long, ByVal Y As Long, ByVal crColor As Long) As Long

' Device context manipulation
Declare Function CreateCompatibleDC Lib "gdi32" (ByVal hdc As Long) As Long
Declare Function DeleteDC Lib "gdi32" (ByVal hdc As Long) As Long
Declare Function DeleteObject Lib "gdi32" (ByVal hObject As Long) As Long
Declare Function GetObject Lib "gdi32" Alias "GetObjectA" (ByVal hObject As Long, ByVal nCount As Long, lpObject As Any) As Long
Declare Function SelectObject Lib "gdi32" (ByVal hdc As Long, ByVal hObject As Long) As Long
Declare Function GetDC Lib "user32" (ByVal hwnd As Long) As Long

' Bitmap manipulation
Declare Function LoadImage Lib "user32" Alias "LoadImageA" (ByVal hInst As Long, ByVal lpsz As String, ByVal un1 As Long, ByVal n1 As Long, ByVal n2 As Long, ByVal un2 As Long) As Long
Declare Function StretchBlt Lib "gdi32" (ByVal hdc As Long, ByVal X As Long, ByVal Y As Long, ByVal nWidth As Long, ByVal nHeight As Long, ByVal hSrcDC As Long, ByVal xSrc As Long, ByVal ySrc As Long, ByVal nSrcWidth As Long, ByVal nSrcHeight As Long, ByVal dwRop As Long) As Long
Declare Function BitBlt Lib "gdi32" (ByVal hDestDC As Long, ByVal X As Long, ByVal Y As Long, ByVal nWidth As Long, ByVal nHeight As Long, ByVal hSrcDC As Long, ByVal xSrc As Long, ByVal ySrc As Long, ByVal dwRop As Long) As Long
Declare Function CreateCompatibleBitmap Lib "gdi32" (ByVal hdc As Long, ByVal nWidth As Long, ByVal nHeight As Long) As Long


' APPLICATION-SPECIFIC DEFINITIONS

' The following definitions will also be used in the main engine ...

' Tile definition
Type tTile
    SourceX As Integer                  ' Position within tilesource
    SourceY As Integer                  ' Position within tilesource
    AniFrames As Byte                   ' Number of frames, 0=No animation frames
    AniDelay As Byte                    ' Delay when showing animated frames
    AniRandom As Integer                ' Propability of tile to start animation (1:aniprop)
    Blocking As Byte                    ' Is this tile blocking movement of characters ?
    RadarColor As Long                  ' Color for tile to display in radar map
End Type

' Tileset definition
Type tTileSet
    FileName As String                  ' Name of tileset
    TileSize As Integer                 ' Size of (quadratic) tiles
    TileCount As Integer                ' Number of tiles in set
    Tile() As tTile                     ' Array of tiles
End Type
Public TileSet As tTileSet

' Map square definition
Type tMapSquare
    TileIndex As Byte                   ' Connection to applying tile
    AniPhase As Byte                    ' Current animation phase, if animated
    LastAniFrame As Long                ' Stored AniFrames of last phase change: Used for timing the animation
End Type

' Map animation definition
Type tTileAniLookup
    IndexX As Byte
    IndexY As Byte
    Used As Boolean
End Type

' Map definition
Type tMap
    FileName As String               ' Full name and path of map
    CurrentX As Byte                 ' Current Position
    CurrentY As Byte                 '
    ExtentX As Byte                  ' Map dimension
    ExtentY As Byte                  '
    DisplayExtentX As Byte           ' Number of tiles to display
    DisplayExtentY As Byte           ' Number of tiles to display
    Squares() As tMapSquare          ' Squares
    AniLookup(99) As tTileAniLookup  ' Index of animated tiles currently visible
End Type
Public Map As tMap

' Application data
Type tProgram
    Initialized As Boolean          ' Already initialized ?
    Running As Boolean              ' Already running (or terminated?)
    EllapsedFrames As Long          ' How many frames so far ?
    Title As String                 ' Display in startup window
End Type
Public Program As tProgram

' The following definitions will not appear in the main engine ...

' Special for editor: Border Sequence, NOTE: This defs are missing in the game
Type tBorderTile
    MasterIndex As Integer
    Border(11) As Integer
End Type
Public BorderTile(19) As tBorderTile
Public BorderTileCount As Integer

' Special for editor: Random Sequence, NOTE: This defs are missing in the game
Type tRandomTile
    TileCount As Integer
    TileIndex(19) As Integer
End Type
Public RandomTile(19) As tRandomTile
Public RandomTileCount As Integer

' Special for editor: Composite Sequence, NOTE: This defs are missing in the game
Type tCompositeTile
    SizeX As Integer
    SizeY As Integer
    TileCount As Integer
    TileIndex(19) As Integer
End Type
Public CompositeTile(19) As tCompositeTile
Public CompositeTileCount As Integer

' Tilesource bitmap definitions, Note: It's all API, for this editor only
Public TileSourceDC As Long, TileSourceBMP As Long

' Backbuffer definition, Note: It's all API, for this editor only
Public BackDC As Long, BackBMP As Long

' Tile currently selected on tile panel
Type tSelectedTile
    Index As Integer
    RandomSequence As Integer
    BorderSequence As Integer
    CompositeSequence As Integer
End Type
Public SelectedTile As tSelectedTile


' ------------------------------------------------------------------------------------
' Loads the tileset data and the tile source bitmap (the same sub is used in the game)
Sub LoadTileset(sName As String)

    On Error GoTo E_LoadTileset
    
    Dim nRunTiles As Integer        ' Variable to run though all tiles found
    
    ' Open tileset definition
    Open sName + ".def" For Input As #1
    
    ' Read tile size and number, calculate tiles to be shown in display (450x450)
    Input #1, TileSet.TileSize
    Map.DisplayExtentX = (fEditor.picMap.Width \ 10) \ TileSet.TileSize
    Map.DisplayExtentY = (fEditor.picMap.Height \ 10) \ TileSet.TileSize
    
    Input #1, TileSet.TileCount
    
    ' Dimension the array holding the tiles
    ReDim TileSet.Tile(TileSet.TileCount)
    
    ' Read all the tile definitions
    For nRunTiles = 0 To TileSet.TileCount - 1
        With TileSet.Tile(nRunTiles)
            Input #1, .SourceX, .SourceY, .AniFrames, .Blocking, .AniDelay, .AniRandom, .RadarColor
        End With
    Next
    
    ' Special for Editor: Read border and randomizing defs, NOTE: This code is missing in the def loader in the game!
    
    Dim nDefType As Integer         ' Variable for reading type of editor def
    
    RandomTileCount = 0
    BorderTileCount = 0
    CompositeTileCount = 0
    
    ' Read rest of def file
    Do While Not EOF(1)
        
        ' Read type of special def
        Input #1, nDefType
        
        ' Read depending on def type...
        Select Case nDefType
        
            ' Bordered tile
            Case 0
            
                ' Input master and border tile indexes
                With BorderTile(BorderTileCount)
                    Input #1, .MasterIndex
                    Input #1, .Border(0), .Border(1), .Border(2), .Border(3), .Border(4), .Border(5), .Border(6), .Border(7), .Border(8), .Border(9), .Border(10), .Border(11)
                End With
                
                ' Increase border sequence counter
                BorderTileCount = BorderTileCount + 1
            
            ' Randomized tile
            Case 1
            
                ' Input random sequence tiles
                With RandomTile(RandomTileCount)
                    Input #1, .TileCount
                    For nRunTiles = 0 To .TileCount - 1
                        Input #1, .TileIndex(nRunTiles)
                    Next
                End With
                
                ' Increase random sequence counter
                RandomTileCount = RandomTileCount + 1
                
             ' Composite tiles
            Case 2
            
                ' Input composite sequence tiles
                With CompositeTile(CompositeTileCount)
                    Input #1, .SizeX
                    Input #1, .SizeY
                    .TileCount = .SizeX * .SizeY
                    For nRunTiles = 0 To .TileCount - 1
                        Input #1, .TileIndex(nRunTiles)
                    Next
                End With
                
                ' Increase composite sequence counter
                CompositeTileCount = CompositeTileCount + 1
                
        End Select
        
    Loop
    
    ' Close tileset definition
    Close #1
    
    ' Load Tilesource bitmap into API DC (Editor only)
    TileSourceBMP = LoadImage(ByVal 0&, sName + ".bmp", 0, 0, 0, LR_LOADFROMFILE)
    SelectObject TileSourceDC, TileSourceBMP
    
    ' Set tileset name
    TileSet.FileName = sName
    
    ' Set Tile panel
    fToolTIles.vscTile.Value = 0
    fToolTIles.vscTile.Max = TileSet.TileCount
        
    Exit Sub
    
E_LoadTileset:

    MsgBox "ERROR: Tileset load failure.", vbCritical + vbOKOnly + vbSystemModal, "Map Editor"
    Unload fEditor
    End
    
End Sub

' ------------------------------------------------------------------------------------
' Initializes the application
Sub AppInit()

    Program.Title = "Map Editor"
    fStartup.Show 1
    
    ' Set initial interface values
    fToolTIles.vscTile.Max = TileSet.TileCount - 6
    SelectedTile.Index = 0
    fEditor.vscMap.Max = 99 - Map.DisplayExtentX
    fEditor.hscMap.Max = 99 - Map.DisplayExtentY
    Call ChangeMapSize(149, 149)
    
    ' Initialize backbuffer
    BackDC = CreateCompatibleDC(fEditor.picMap.hdc)
    BackBMP = CreateCompatibleBitmap(fEditor.picMap.hdc, 1024, 768)
    SelectObject BackDC, BackBMP
    TileSourceDC = CreateCompatibleDC(fEditor.picMap.hdc)
    ' Load the tileset
    Call LoadTileset(App.Path + "\..\common files\Default")

    ' Reset selected tile settings
    SelectedTile.Index = 0
    SelectedTile.BorderSequence = GetTileBorderSequence(SelectedTile.Index)
    SelectedTile.RandomSequence = GetTileRandomSequence(SelectedTile.Index)
    SelectedTile.CompositeSequence = GetTileCompositeSequence(SelectedTile.Index)

End Sub

' ------------------------------------------------------------------------------------
' Cleans up, most important: Delete all api objects
Sub AppExit()

    ' Unload tool windows
    Unload fToolProps
    Unload fToolTIles
    Unload fToolRadar
    
    ' Delete tile source api objects
    DeleteDC TileSourceDC
    DeleteObject TileSourceBMP
    
    ' Delete backbuffer api objects
    DeleteDC BackDC
    DeleteObject BackBMP
    
    ' Unload editor
    Unload fEditor
    
End Sub

' ------------------------------------------------------------------------------------
' Application main loop, does redrawing etc., called when main form is activated
Sub AppLoop()
    
    Dim nLoopDelay As Double        ' Used to time Frame delay
    
    ' Application started - status
    Program.Initialized = True
    
    ' Redraw the interface for the first time
    Call DrawMap
    Call DrawTilePanel
    
    ' Execute main loop
    Program.Running = True
    
    Do
    
        ' Calculate frame duration
        nLoopDelay = Timer + 0.05
    
        ' Increase Global Frame Counter
        Program.EllapsedFrames = Program.EllapsedFrames + 1
        
        ' Calculate animated map tiles
        Call ProcessAnimatedTiles
        
        ' Display map contents
        FlipBuffers
        
        ' Receive user input
        DoEvents
        
        ' This loop times the frames to last exactly for 1/20 second.
        ' If longer frame duration occures, it is simply ignored (you may experience some unsmoothness, thats all)
        Do While nLoopDelay > Timer
        Loop
    
    ' Terminate if any procedure sets app status to "terminated"
    
    Loop While Program.Running
    
    ' Clean up application
    Call AppExit
    
End Sub
Sub ChangeMapSize(nX As Byte, nY As Byte)

    Map.ExtentX = nX
    Map.ExtentY = nY
    ReDim Map.Squares(Map.ExtentX, Map.ExtentY)
    
    fEditor.hscMap.Value = 0
    fEditor.hscMap.Max = Map.ExtentX - Map.DisplayExtentX
    fEditor.vscMap.Max = Map.ExtentY - Map.DisplayExtentY

End Sub

' ------------------------------------------------------------------------------------
' Draws the map fragment currently visible to the backbuffer
Sub DrawMap()

    Dim nRunX As Integer, nRunY As Integer          ' Variables to run through the map squares to be drawn
    
    ' Run through the visible map area
    For nRunX = 0 To Map.DisplayExtentX
        For nRunY = 0 To Map.DisplayExtentY
            Call DrawSquare(nRunX, nRunY)
        Next
    Next

End Sub

' ------------------------------------------------------------------------------------
' Draws the map square given to the backbuffer
Sub DrawSquare(nPosX As Integer, nPosY As Integer)

    Dim nCurDstX As Integer, nCurDstY As Integer    ' Current blit destination within backbuffer
    Dim nCurTileIndex As Integer                    ' Index of tile being processed
    Dim nCurSrcX As Integer, nCurSrcY As Integer    ' Current blit source withing tilesource bitmap
    
    ' Calculate Index of tile being processed
    nCurTileIndex = Map.Squares(nPosX + Map.CurrentX, nPosY + Map.CurrentY).TileIndex
    
    ' Calculate source of tile within tilesource bitmap
    If Map.Squares(nPosX + Map.CurrentX, nPosY + Map.CurrentY).AniPhase = 0 Then
        nCurSrcX = TileSet.Tile(nCurTileIndex).SourceX
        nCurSrcY = TileSet.Tile(nCurTileIndex).SourceY
    Else
        nCurSrcX = TileSet.Tile(nCurTileIndex).SourceX + Map.Squares(nPosX + Map.CurrentX, nPosY + Map.CurrentY).AniPhase * TileSet.TileSize
        nCurSrcY = TileSet.Tile(nCurTileIndex).SourceY
    End If
    
    ' Calculate destination of tile withing backbuffer
    nCurDstX = nPosX * TileSet.TileSize
    nCurDstY = nPosY * TileSet.TileSize
    
    ' Blit to backbuffer
    BitBlt BackDC, nCurDstX, nCurDstY, TileSet.TileSize, TileSet.TileSize, TileSourceDC, nCurSrcX, nCurSrcY, SRCCOPY

End Sub


' ------------------------------------------------------------------------------------
' Draws the tiles currently visible to the tile selection panel
Sub DrawTilePanel()

    Dim nRunTiles As Integer                        ' Variable to run through the tiles to be drawn
    Dim nCurSrcX As Integer, nCurSrcY As Integer    ' Current blit source withing tilesource bitmap
    Dim sFunctionDisplay As String                  ' Holds display string for special functions
    
    ' Run through the tile panel
    For nRunTiles = 0 To 4
            
        ' Calculate source of tile within tilesource bitmap
        nCurSrcX = TileSet.Tile(nRunTiles + fToolTIles.vscTile.Value).SourceX
        nCurSrcY = TileSet.Tile(nRunTiles + fToolTIles.vscTile.Value).SourceY
            
        ' Blit to display currently selected
        BitBlt fToolTIles.picTile(nRunTiles).hdc, 0, 0, TileSet.TileSize, TileSet.TileSize, TileSourceDC, nCurSrcX, nCurSrcY, SRCCOPY

        ' Toggle displays of special functions
        
        sFunctionDisplay = ""
        
        If TileSet.Tile(nRunTiles + fToolTIles.vscTile.Value).AniFrames > 0 Then
            sFunctionDisplay = sFunctionDisplay + "A"
        End If
        
        If GetTileBorderSequence(nRunTiles + fToolTIles.vscTile.Value) > -1 Then
            sFunctionDisplay = sFunctionDisplay + "B"
        End If
        
        If GetTileRandomSequence(nRunTiles + fToolTIles.vscTile.Value) > -1 Then
            sFunctionDisplay = sFunctionDisplay + "R"
        End If
        
        If GetTileCompositeSequence(nRunTiles + fToolTIles.vscTile.Value) > -1 Then
            sFunctionDisplay = sFunctionDisplay + "C"
        End If
        
        fToolTIles.lblFunction(nRunTiles).Caption = sFunctionDisplay
        
    Next

End Sub
' ------------------------------------------------------------------------------------
' Blit the backbuffer to the front buffer
Sub FlipBuffers()

    ' Blit backbuffer to frontbuffer
    BitBlt fEditor.picMap.hdc, 0, 0, Map.DisplayExtentX * TileSet.TileSize, Map.DisplayExtentY * TileSet.TileSize, BackDC, 0, 0, SRCCOPY
    
End Sub

' ------------------------------------------------------------------------------------
' Draws a border around a map square designated to be master of a border sequence
Sub SetBorders(nMapX As Integer, nMapy As Integer, nSequence As Integer)

    Dim nMasterIndex As Integer             ' Index of the square that calls this
    Dim bNB(7) As Integer                   ' Neighbour tiles of square being checked
    Dim nRunNB As Integer                   ' Variable running through all the neighbours
    Dim nOffsetX(7) As Integer              ' Offset constant for checking neighbours
    Dim nOffsetY(7) As Integer              ' Offset constant for checking neighbours
    
    ' Define border offsets
    nOffsetX(0) = 0
    nOffsetY(0) = -1
    nOffsetX(1) = 1
    nOffsetY(1) = -1
    nOffsetX(2) = 1
    nOffsetY(2) = 0
    nOffsetX(3) = 1
    nOffsetY(3) = 1
    nOffsetX(4) = 0
    nOffsetY(4) = 1
    nOffsetX(5) = -1
    nOffsetY(5) = 1
    nOffsetX(6) = -1
    nOffsetY(6) = 0
    nOffsetX(7) = -1
    nOffsetY(7) = -1
    
    ' Get master index
    nMasterIndex = Map.Squares(nMapX, nMapy).TileIndex
    
    ' Read all the neighbours of the tile ...
    For nRunNB = 0 To 7
    
        If (nMapX + nOffsetX(nRunNB) > Map.ExtentX) Or (nMapX + nOffsetX(nRunNB) < 0) Or (nMapy + nOffsetY(nRunNB) > Map.ExtentY) Or (nMapy + nOffsetY(nRunNB) < 0) Then
            ' Consider map edge to be a master tile
            bNB(nRunNB) = 0
        Else
            If (Map.Squares(nMapX + nOffsetX(nRunNB), nMapy + nOffsetY(nRunNB)).TileIndex = BorderTile(nSequence).MasterIndex) Then
                ' On a tile that is master of the same sequence: Do not draw at all
                bNB(nRunNB) = 0
            Else
                ' All other tiles: Try to match
                bNB(nRunNB) = 1
            End If
        End If
        
    Next
            
    ' Calculate all borders (This is quite crude: Using a lookup table an a recursion, you should be able to do it an more ellegant way ...
    
    ' Tile N
    If bNB(0) = 1 Then
        With Map.Squares(nMapX + nOffsetX(0), nMapy + nOffsetY(0))
            If Not (bNB(7) = 0 Or bNB(1) = 0) Then
                .TileIndex = BorderTile(nSequence).Border(0)
            Else
                If bNB(1) = 0 Then
                    .TileIndex = BorderTile(nSequence).Border(11)
                End If
                If bNB(7) = 0 Then
                    .TileIndex = BorderTile(nSequence).Border(2)
                End If
            End If
        End With
    End If
    
    ' Tile NE
    If bNB(1) = 1 Then
        With Map.Squares(nMapX + nOffsetX(1), nMapy + nOffsetY(1))
            If bNB(0) = 1 And bNB(2) = 1 Then
                .TileIndex = BorderTile(nSequence).Border(1)
            End If
            If bNB(0) = 0 And bNB(2) = 0 Then
                .TileIndex = BorderTile(nSequence).Border(2)
            End If
        End With
    End If
    
    ' Tile E
    If bNB(2) = 1 Then
        With Map.Squares(nMapX + nOffsetX(2), nMapy + nOffsetY(2))
            If Not (bNB(1) = 0 Or bNB(3) = 0) Then
                .TileIndex = BorderTile(nSequence).Border(3)
            Else
                If bNB(1) = 0 Then
                    .TileIndex = BorderTile(nSequence).Border(5)
                End If
                If bNB(3) = 0 Then
                    .TileIndex = BorderTile(nSequence).Border(2)
                End If
            End If
       End With
    End If
   
    ' Tile SE
    If bNB(3) = 1 Then
        With Map.Squares(nMapX + nOffsetX(3), nMapy + nOffsetY(3))
            If bNB(2) = 1 And bNB(4) = 1 Then
                .TileIndex = BorderTile(nSequence).Border(4)
            End If
            If bNB(2) = 0 And bNB(4) = 0 Then
                .TileIndex = BorderTile(nSequence).Border(5)
            End If
        End With
    End If
    
    ' Tile S
    If bNB(4) = 1 Then
        With Map.Squares(nMapX + nOffsetX(4), nMapy + nOffsetY(4))
            If Not (bNB(3) = 0 Or bNB(5) = 0) Then
                .TileIndex = BorderTile(nSequence).Border(6)
            Else
                If bNB(3) = 0 Then
                    .TileIndex = BorderTile(nSequence).Border(8)
                End If
                If bNB(5) = 0 Then
                    .TileIndex = BorderTile(nSequence).Border(5)
                End If
            End If
        End With
    End If
    
    ' Tile SW
    If bNB(5) = 1 Then
        With Map.Squares(nMapX + nOffsetX(5), nMapy + nOffsetY(5))
            If bNB(4) = 1 And bNB(6) = 1 Then
                .TileIndex = BorderTile(nSequence).Border(7)
            End If
            If bNB(4) = 0 And bNB(6) = 0 Then
                .TileIndex = BorderTile(nSequence).Border(8)
            End If
        End With
    End If
    
    ' Tile W
    If bNB(6) = 1 Then
        With Map.Squares(nMapX + nOffsetX(6), nMapy + nOffsetY(6))
            If Not (bNB(7) = 0 Or bNB(5) = 0) Then
                .TileIndex = BorderTile(nSequence).Border(9)
            Else
                If bNB(5) = 0 Then
                    .TileIndex = BorderTile(nSequence).Border(11)
                End If
                If bNB(7) = 0 Then
                    .TileIndex = BorderTile(nSequence).Border(8)
                End If
            End If
        End With
    End If
    
    ' Tile NW
    If bNB(7) = 1 Then
        With Map.Squares(nMapX + nOffsetX(7), nMapy + nOffsetY(7))
            If bNB(0) = 1 And bNB(6) = 1 Then
                .TileIndex = BorderTile(nSequence).Border(10)
            End If
            If bNB(0) = 0 And bNB(6) = 0 Then
                .TileIndex = BorderTile(nSequence).Border(11)
            End If
        End With
    End If
    
End Sub

' ------------------------------------------------------------------------------------
' Fills the lookup table of animated map tiles
Sub FillMapAnimationLookup()

    Dim nRunX As Integer        ' Runs through visible map tiles
    Dim nRunY As Integer        ' Runs through visible map tiles
    
    Dim nLookupCount As Integer ' Counter for tiles to add
    
    ' Clear lookup table
    For nLookupCount = 0 To 99
        Map.AniLookup(nLookupCount).Used = False
    Next
    
    ' Create lookuptable ...
    nLookupCount = 0
    
    ' Run through visible map squares
    For nRunX = 0 To Map.DisplayExtentX
        For nRunY = 0 To Map.DisplayExtentY
        
            ' Add tile if it is animated
            If TileSet.Tile(Map.Squares(nRunX + Map.CurrentX, nRunY + Map.CurrentY).TileIndex).AniFrames > 0 Then
                Map.AniLookup(nLookupCount).IndexX = Map.CurrentX + nRunX
                Map.AniLookup(nLookupCount).IndexY = Map.CurrentY + nRunY
                Map.AniLookup(nLookupCount).Used = True
                nLookupCount = nLookupCount + 1
                If nLookupCount = 100 Then Exit Sub
            End If
            
        Next
    Next
    
End Sub
' ------------------------------------------------------------------------------------
' Draws and calculates animated Tiles, about the same routine will be used in the game
Sub ProcessAnimatedTiles()

    Dim nRunTiles As Integer    ' Runs through all animated tiles
    
    ' Do not calculate at all if no animated tiles are found
    If Map.AniLookup(0).Used = False Then Exit Sub
    
    Do
        ' Animate tile
        With Map.Squares(Map.AniLookup(nRunTiles).IndexX, Map.AniLookup(nRunTiles).IndexY)
            
            ' No animation in progress? Check for animation start!
            If .AniPhase = 0 Then
                
                If Int(Rnd(1) * TileSet.Tile(.TileIndex).AniRandom) + 1 = 1 Then
                    .AniPhase = 1
                    .LastAniFrame = Program.EllapsedFrames
                    Call DrawSquare(Map.AniLookup(nRunTiles).IndexX - Map.CurrentX, Map.AniLookup(nRunTiles).IndexY - Map.CurrentY)
                End If
                
            ' Animation in progress? Check for next animation phase and finish!
            Else
                
                ' Compare if next animation phase is reached
                If Program.EllapsedFrames = .LastAniFrame + TileSet.Tile(.TileIndex).AniDelay Then
                    
                    ' Yes? Display next animation phase
                    .AniPhase = .AniPhase + 1
                    .LastAniFrame = Program.EllapsedFrames
                    
                    ' Last phase of animation? Cancel animation of this tile.
                    If .AniPhase > TileSet.Tile(.TileIndex).AniFrames Then
                        .AniPhase = 0
                    End If
                    
                    Call DrawSquare(Map.AniLookup(nRunTiles).IndexX - Map.CurrentX, Map.AniLookup(nRunTiles).IndexY - Map.CurrentY)
                    
                End If
                
            End If
            
        End With
        
        ' Increase lookup counter, finish if first unused tile is encountered or max tile number reached
        nRunTiles = nRunTiles + 1
        
    Loop Until Map.AniLookup(nRunTiles).Used = False Or nRunTiles = 99
    
End Sub

' ------------------------------------------------------------------------------------
' Look if tile has border ...
Function GetTileBorderSequence(nTileIndex As Integer) As Integer

    Dim nRunSequence As Integer         ' Variable to run through all sequences
    Dim nRunTiles As Integer            ' Variable to run through all tiles within a sequence

    GetTileBorderSequence = -1
    
    For nRunSequence = 0 To BorderTileCount - 1
       If BorderTile(nRunSequence).MasterIndex = nTileIndex Then
           GetTileBorderSequence = nRunSequence
           Exit Function
       End If
    Next

End Function
   
' ------------------------------------------------------------------------------------
' Look if tile has random sequence ...
Function GetTileRandomSequence(nTileIndex As Integer) As Integer
    
    Dim nRunSequence As Integer         ' Variable to run through all sequences
    Dim nRunTiles As Integer            ' Variable to run through all tiles within a sequence

    GetTileRandomSequence = -1
   
    For nRunSequence = 0 To RandomTileCount - 1
        For nRunTiles = 0 To RandomTile(nRunSequence).TileCount - 1
            If RandomTile(nRunSequence).TileIndex(nRunTiles) = nTileIndex Then
                GetTileRandomSequence = nRunSequence
                Exit Function
            End If
        Next
    Next

End Function

' ------------------------------------------------------------------------------------
' Look if tile has composite sequence ...
Function GetTileCompositeSequence(nTileIndex As Integer) As Integer

    Dim nRunSequence As Integer         ' Variable to run through all sequences
    Dim nRunTiles As Integer            ' Variable to run through all tiles within a sequence

    GetTileCompositeSequence = -1
    
    For nRunSequence = 0 To CompositeTileCount - 1
        For nRunTiles = 0 To CompositeTile(nRunSequence).TileCount - 1
            If CompositeTile(nRunSequence).TileIndex(nRunTiles) = nTileIndex Then
                GetTileCompositeSequence = nRunSequence
                Exit Function
            End If
        Next
    Next

End Function

' ------------------------------------------------------------------------------------
' Display the "radar" overview map

Sub DrawRadar()

    Dim nRunX As Integer, nRunY As Integer  ' Variables to run through all squares
    
    ' Draw overview using API SetPixelV
    For nRunX = 0 To Map.ExtentX
        For nRunY = 0 To Map.ExtentY
            SetPixelV fToolRadar.picRadar.hdc, nRunX + 3, nRunY + 3, TileSet.Tile(Map.Squares(nRunX, nRunY).TileIndex).RadarColor
        Next
    Next

    ' Draw visible rectangle using standard line
    fToolRadar.picRadar.Line (Map.CurrentX + 3, Map.CurrentY + 3)-(Map.CurrentX + 3 + Map.DisplayExtentX, Map.CurrentY + 3 + Map.DisplayExtentY), RGB(250, 250, 250), B
    
End Sub

' ------------------------------------------------------------------------------------
' Loads a map into editor
Sub OpenMap()

    Dim nRunX As Integer, nRunY As Integer      ' Variables to run through all squares
    Dim sLineExp As String                      ' String holding line to be written
    
    ' Query before erasing
    If MsgBox("This operation will dump the current map. Proceed?", vbYesNo + vbQuestion + vbSystemModal, "Map Editor") = vbNo Then Exit Sub

    ' Hide tool windows
    Unload fToolTIles
    Unload fToolProps

    ' Set common dialog propertys
    With fEditor.cdgEditor
        .DialogTitle = "Open Map File"
        .Filter = "Map File (*.MAP)|*.MAP"
        .CancelError = True
        .InitDir = App.Path
        On Error GoTo NoOpen
        .ShowOpen
        On Error GoTo 0
    End With
    
    ' Open the map ...
    
    On Error GoTo E_OpenMap
    
    Screen.MousePointer = 11
    fEditor.Refresh
    
    ' Change map filename
    Map.FileName = fEditor.cdgEditor.FileName
    
    ' Open output file
    Open fEditor.cdgEditor.FileName For Input As #1
    
    ' Input general data
    Input #1, TileSet.FileName
    Input #1, Map.FileName
    Input #1, Map.ExtentX, Map.ExtentY
    Call ChangeMapSize(Map.ExtentX, Map.ExtentY)

    ' Input squares
    For nRunY = 0 To Map.ExtentY
        Input #1, sLineExp
        For nRunX = 0 To Map.ExtentX
            Map.Squares(nRunX, nRunY).TileIndex = Val(Mid(sLineExp, nRunX * 3 + 1, 3))
        Next
    Next
    
    ' Close input file
    Close #1
    
    ' Update display
    Call LoadTileset(TileSet.FileName)
    Call DrawMap
    Call FillMapAnimationLookup
    
NoOpen:

    Screen.MousePointer = 0
    Exit Sub
    
E_OpenMap:
    
    MsgBox "ERROR: Map load failure.", vbCritical + vbOKOnly + vbSystemModal, "Map Editor"
    Screen.MousePointer = 0
    
End Sub

' ------------------------------------------------------------------------------------
' Clears the map
Sub ClearMap()

    Dim nRunX As Integer, nRunY As Integer      ' Variables to run through all squares
    
    ' Query before erasing
    If MsgBox("New Map will be created based on tile currently selected. Proceed?", vbYesNo + vbQuestion + vbSystemModal, "Map Editor") = vbYes Then
    
        ' Hide tool windows
        fToolTIles.Hide
        fEditor.tbrEditor.Buttons("TOOLPAL").Value = tbrUnpressed
        fToolProps.Hide
        fEditor.tbrEditor.Buttons("TOOLPROP").Value = tbrUnpressed
        
        ' Clear map
        For nRunX = 0 To Map.ExtentX
            For nRunY = 0 To Map.ExtentY
                If SelectedTile.RandomSequence > -1 Then
                    Map.Squares(nRunX, nRunY).TileIndex = RandomTile(SelectedTile.RandomSequence).TileIndex(Int(Rnd(1) * RandomTile(SelectedTile.RandomSequence).TileCount))
                Else
                    Map.Squares(nRunX, nRunY).TileIndex = SelectedTile.Index
                End If
            Next
        Next
        
        ' Refresh map
        Call DrawMap
        Call FillMapAnimationLookup
    
    End If
    
End Sub

' ------------------------------------------------------------------------------------
' Saves the map
Sub SaveMap()

    Dim nRunX As Integer, nRunY As Integer      ' Variables to run through all squares
    Dim sLineExp As String                      ' String holding line to be written
    
    ' Hide tool windows
    fToolTIles.Hide
    fToolProps.Hide
    
    ' Set common dialog propertys, get filename
    With fEditor.cdgEditor
        .DialogTitle = "Save Map File"
        .Filter = "Map File (*.MAP)|*.MAP"
        .CancelError = True
        .InitDir = App.Path
        On Error GoTo NoSave
        .ShowSave
        On Error GoTo 0
    End With
        
    ' Save the map ...
    
    Screen.MousePointer = 11
    fEditor.Refresh
    
    ' Change map filename
    Map.FileName = fEditor.cdgEditor.FileName
    
    ' Open output file
    Open fEditor.cdgEditor.FileName For Output As #1
    
    ' Write general data
    Print #1, TileSet.FileName
    Print #1, Map.FileName
    Print #1, Map.ExtentX, Map.ExtentY
    
    ' Write squares
    For nRunY = 0 To Map.ExtentY
        sLineExp = ""
        For nRunX = 0 To Map.ExtentX
            sLineExp = sLineExp + Format(Map.Squares(nRunX, nRunY).TileIndex, "000")
        Next
        Print #1, sLineExp
    Next
    
    ' Close output file
    Close #1
    
NoSave:

    Screen.MousePointer = 0
    
End Sub

