Attribute VB_Name = "mEditor"
' --------------------------------------------------------------------------------------
' THIS MODULE CONTAINS CODE SPECIFIC FOR THE TILESET EDITOR.
' REPLACE AT WILL, BUT NOTE: THE ORIGINAL DIRECTXSUITE APPLICATION MAY NO LONGER
' WORK WITH MAPS GENERATED USING YOUR MODIFIED EDITOR.
' --------------------------------------------------------------------------------------

' NOTE: This App does not use dirty rectangles or any of the
'       fancy tricks a real game engine needs. It's just a dumb editor.
'       For the real cool stuff, check out the engine sample.

Option Explicit


' API DECLARATIONS

' Constants and types for use with win32 API
Const IMAGE_BITMAP = 0
Const LR_LOADFROMFILE = &H10
Const SRCCOPY = &HCC0020

' Window declarations
Declare Function SetWindowPos Lib "user32" (ByVal hwnd As Long, ByVal hWndInsertAfter As Long, ByVal X As Long, ByVal Y As Long, ByVal cx As Long, ByVal cy As Long, ByVal wFlags As Long) As Long

' Single Pixel manipulation
Declare Function GetPixel Lib "gdi32" (ByVal hdc As Long, ByVal X As Long, ByVal Y As Long) As Long
Declare Function SetPixelV Lib "gdi32" (ByVal hdc As Long, ByVal X As Long, ByVal Y As Long, ByVal crColor As Long) As Long

' Device context manipulation
Declare Function CreateCompatibleDC Lib "gdi32" (ByVal hdc As Long) As Long
Declare Function DeleteDC Lib "gdi32" (ByVal hdc As Long) As Long
Declare Function DeleteObject Lib "gdi32" (ByVal hObject As Long) As Long
Declare Function GetObject Lib "gdi32" Alias "GetObjectA" (ByVal hObject As Long, ByVal nCount As Long, lpObject As Any) As Long
Declare Function SelectObject Lib "gdi32" (ByVal hdc As Long, ByVal hObject As Long) As Long
Declare Function GetDC Lib "user32" (ByVal hwnd As Long) As Long

' Bitmap manipulation
Declare Function LoadImage Lib "user32" Alias "LoadImageA" (ByVal hInst As Long, ByVal lpsz As String, ByVal un1 As Long, ByVal n1 As Long, ByVal n2 As Long, ByVal un2 As Long) As Long
Declare Function StretchBlt Lib "gdi32" (ByVal hdc As Long, ByVal X As Long, ByVal Y As Long, ByVal nWidth As Long, ByVal nHeight As Long, ByVal hSrcDC As Long, ByVal xSrc As Long, ByVal ySrc As Long, ByVal nSrcWidth As Long, ByVal nSrcHeight As Long, ByVal dwRop As Long) As Long
Declare Function BitBlt Lib "gdi32" (ByVal hDestDC As Long, ByVal X As Long, ByVal Y As Long, ByVal nWidth As Long, ByVal nHeight As Long, ByVal hSrcDC As Long, ByVal xSrc As Long, ByVal ySrc As Long, ByVal dwRop As Long) As Long
Declare Function CreateCompatibleBitmap Lib "gdi32" (ByVal hdc As Long, ByVal nWidth As Long, ByVal nHeight As Long) As Long


' APPLICATION-SPECIFIC DEFINITIONS

' The following definitions will also be used in the main engine ...

' Tile definition
Type tTile
    SourceX As Integer                  ' Position within tilesource
    SourceY As Integer                  ' Position within tilesource
    AniFrames As Byte                   ' Number of frames, 0=No animation frames
    AniDelay As Byte                    ' Delay when showing animated frames
    AniRandom As Integer                ' Propability of tile to start animation (1:aniprop)
    Blocking As Byte                    ' Is this tile blocking movement of characters ?
    RadarColor As Long                  ' Color for tile to display in radar map
End Type

' Tileset definition
Type tTileSet
    FileName As String                  ' Name of tileset
    TileSize As Integer                 ' Size of (quadratic) tiles
    TileCount As Integer                ' Number of tiles in set
    Tile() As tTile                     ' Array of tiles
End Type
Public TileSet As tTileSet

' Application data
Type tProgram
    Initialized As Boolean          ' Already initialized ?
    Running As Boolean              ' Already running (or terminated?)
    EllapsedFrames As Long          ' How many frames so far ?
    Title As String                 ' Display in startup window
End Type
Public Program As tProgram


' The following definitions will not appear in the main engine ...

' Special for editor: Border Sequence, NOTE: This defs are missing in the game
Type tBorderTile
    MasterIndex As Integer
    Border(11) As Integer
End Type
Public BorderTile(19) As tBorderTile
Public BorderTileCount As Integer

' Special for editor: Random Sequence, NOTE: This defs are missing in the game
Type tRandomTile
    TileCount As Integer
    TileIndex(19) As Integer
End Type
Public RandomTile(19) As tRandomTile
Public RandomTileCount As Integer

' Special for editor: Composite Sequence, NOTE: This defs are missing in the game
Type tCompositeTile
    SizeX As Integer
    SizeY As Integer
    TileCount As Integer
    TileIndex(19) As Integer
End Type
Public CompositeTile(19) As tCompositeTile
Public CompositeTileCount As Integer

' Tilesource bitmap definitions, Note: It's all API, for this editor only
Public TileSourceDC As Long, TileSourceBMP As Long

' Backbuffer definition, Note: It's all API, for this editor only
Public BackDC As Long, BackBMP As Long

' Tile currently selected on tile panel
Type tSelectedTile
    Index As Integer
    RandomSequence As Integer
    BorderSequence As Integer
    CompositeSequence As Integer
End Type
Public SelectedTile As tSelectedTile


' ------------------------------------------------------------------------------------
' Loads the tileset data and the tile source bitmap (the same sub is used in the game)
Sub LoadTileset(sName As String)

    On Error GoTo E_LoadTileset
    
    Dim nRunTiles As Integer        ' Variable to run though all tiles found
    
    ' Open tileset definition
    Open sName + ".def" For Input As #1
    
    ' Read tile size and number, calculate tiles to be shown in display (450x450)
    Input #1, TileSet.TileSize
    
    Input #1, TileSet.TileCount
    
    ' Dimension the array holding the tiles
    ReDim TileSet.Tile(TileSet.TileCount)
    
    ' Read all the tile definitions
    For nRunTiles = 0 To TileSet.TileCount - 1
        With TileSet.Tile(nRunTiles)
            Input #1, .SourceX, .SourceY, .AniFrames, .Blocking, .AniDelay, .AniRandom, .RadarColor
        End With
    Next
    
    ' Special for Editor: Read border and randomizing defs, NOTE: This code is missing in the def loader in the game!
    
    Dim nDefType As Integer         ' Variable for reading type of editor def
    
    RandomTileCount = 0
    BorderTileCount = 0
    CompositeTileCount = 0
    
    ' Read rest of def file
    Do While Not EOF(1)
        
        ' Read type of special def
        Input #1, nDefType
        
        ' Read depending on def type...
        Select Case nDefType
        
            ' Bordered tile
            Case 0
            
                ' Input master and border tile indexes
                With BorderTile(BorderTileCount)
                    Input #1, .MasterIndex
                    Input #1, .Border(0), .Border(1), .Border(2), .Border(3), .Border(4), .Border(5), .Border(6), .Border(7), .Border(8), .Border(9), .Border(10), .Border(11)
                End With
                
                ' Increase border sequence counter
                BorderTileCount = BorderTileCount + 1
            
            ' Randomized tile
            Case 1
            
                ' Input random sequence tiles
                With RandomTile(RandomTileCount)
                    Input #1, .TileCount
                    For nRunTiles = 0 To .TileCount - 1
                        Input #1, .TileIndex(nRunTiles)
                    Next
                End With
                
                ' Increase random sequence counter
                RandomTileCount = RandomTileCount + 1
                
             ' Composite tiles
            Case 2
            
                ' Input composite sequence tiles
                With CompositeTile(CompositeTileCount)
                    Input #1, .SizeX
                    Input #1, .SizeY
                    .TileCount = .SizeX * .SizeY
                    For nRunTiles = 0 To .TileCount - 1
                        Input #1, .TileIndex(nRunTiles)
                    Next
                End With
                
                ' Increase composite sequence counter
                CompositeTileCount = CompositeTileCount + 1
                
        End Select
        
    Loop
    
    ' Close tileset definition
    Close #1
    
    ' Load Tilesource bitmap into API DC (Editor only)
    TileSourceBMP = LoadImage(ByVal 0&, sName + ".bmp", 0, 0, 0, LR_LOADFROMFILE)
    SelectObject TileSourceDC, TileSourceBMP
    
    ' Set tileset name
    TileSet.FileName = sName
    
    Exit Sub
    
E_LoadTileset:

    MsgBox "ERROR: Tileset load failure.", vbCritical + vbOKOnly + vbSystemModal, "Tileset Editor"
    Unload fEditor
    End
    
End Sub

' ------------------------------------------------------------------------------------
' Initializes the application
Sub AppInit()

    Program.Title = "Tileset Editor"
    fStartup.Show 1
    
    ' Initialize backbuffer
    BackDC = CreateCompatibleDC(fEditor.picImage.hdc)
    BackBMP = CreateCompatibleBitmap(fEditor.picImage.hdc, 1024, 768)
    SelectObject BackDC, BackBMP
    TileSourceDC = CreateCompatibleDC(fEditor.picImage.hdc)
    
    ' Load the tileset
    Call LoadTileset(App.Path + "\..\common files\Default")

    ' Reset selected tile settings
    SelectedTile.Index = 0
    SelectedTile.BorderSequence = GetTileBorderSequence(SelectedTile.Index)
    SelectedTile.RandomSequence = GetTileRandomSequence(SelectedTile.Index)
    SelectedTile.CompositeSequence = GetTileCompositeSequence(SelectedTile.Index)

End Sub

' ------------------------------------------------------------------------------------
' Cleans up, most important: Delete all api objects
Sub AppExit()
    
    ' Delete tile source api objects
    DeleteDC TileSourceDC
    DeleteObject TileSourceBMP
    
    ' Delete backbuffer api objects
    DeleteDC BackDC
    DeleteObject BackBMP
    
    ' Unload editor
    Unload fEditor
    
End Sub

' ------------------------------------------------------------------------------------
' Application main loop, does redrawing etc., called when main form is activated
Sub AppLoop()
    
    Dim nLoopDelay As Double        ' Used to time Frame delay
    
    ' Application started - status
    Program.Initialized = True
    
    ' Execute main loop
    Program.Running = True
    
    Do
    
        ' Calculate frame duration
        nLoopDelay = Timer + 0.05
    
        ' Increase Global Frame Counter
        Program.EllapsedFrames = Program.EllapsedFrames + 1
        
        
        ' Receive user input
        DoEvents
        
        ' This loop times the frames to last exactly for 1/20 second.
        ' If longer frame duration occures, it is simply ignored (you may experience some unsmoothness, thats all)
        Do While nLoopDelay > Timer
        Loop
    
        ' Terminate if any procedure sets app status to "terminated"
    
    Loop While Program.Running
    
    ' Clean up application
    Call AppExit
    
End Sub

' ------------------------------------------------------------------------------------
' Look if tile has border ...
Function GetTileBorderSequence(nTileIndex As Integer) As Integer

    Dim nRunSequence As Integer         ' Variable to run through all sequences
    Dim nRunTiles As Integer            ' Variable to run through all tiles within a sequence

    GetTileBorderSequence = -1
    
    For nRunSequence = 0 To BorderTileCount - 1
       If BorderTile(nRunSequence).MasterIndex = nTileIndex Then
           GetTileBorderSequence = nRunSequence
           Exit Function
       End If
    Next

End Function
   
' ------------------------------------------------------------------------------------
' Look if tile has random sequence ...
Function GetTileRandomSequence(nTileIndex As Integer) As Integer
    
    Dim nRunSequence As Integer         ' Variable to run through all sequences
    Dim nRunTiles As Integer            ' Variable to run through all tiles within a sequence

    GetTileRandomSequence = -1
   
    For nRunSequence = 0 To RandomTileCount - 1
        For nRunTiles = 0 To RandomTile(nRunSequence).TileCount - 1
            If RandomTile(nRunSequence).TileIndex(nRunTiles) = nTileIndex Then
                GetTileRandomSequence = nRunSequence
                Exit Function
            End If
        Next
    Next

End Function

' ------------------------------------------------------------------------------------
' Look if tile has composite sequence ...
Function GetTileCompositeSequence(nTileIndex As Integer) As Integer

    Dim nRunSequence As Integer         ' Variable to run through all sequences
    Dim nRunTiles As Integer            ' Variable to run through all tiles within a sequence

    GetTileCompositeSequence = -1
    
    For nRunSequence = 0 To CompositeTileCount - 1
        For nRunTiles = 0 To CompositeTile(nRunSequence).TileCount - 1
            If CompositeTile(nRunSequence).TileIndex(nRunTiles) = nTileIndex Then
                GetTileCompositeSequence = nRunSequence
                Exit Function
            End If
        Next
    Next

End Function

